/*

# セキュリティ・キャンプ 2022「OS 自作ゼミ」応募課題

このファイルには、作りかけのリングバッファの実装があります。
このままコンパイルして実行することはできますが、実装が不足しているため、テストに失敗するようです。
欠けている部分を実装して、テストに通るリングバッファを実装してください。


## コンパイル＆実行方法

初期状態のコードをコンパイルして実行すると、以下のような出力が得られます。
```
$ gcc -Wall -Wextra ring.c
$ ./a.out
NG: want 2, got 0
NG: want 3, got 0
FAILED: 2/2 tests failed
```
実装を追加してテストに通るようになると、出力は以下のようになります。
```
$ ./a.out
OK: want 2, got 2
OK: want 3, got 3
PASSED: All tests finished succesfully
```
上記のように、テストにパスするプログラムを書くことが目標です。

## 注意事項

- 講師は GCC あるいは Clang で動作を確認します。
- 最低限、付属の 2 つのテストケースがパスするまで実装してから提出してください。
- 提出の際は本コメントブロックを取り除いてかまいません。


## 解答のヒント

- まずは付属のテストケースが動くようにしてみましょう。
- 付属のテストケースが動くようになったら、テストケースを追加してみましょう。
  - どのようなテストを追加すると、この実装の信頼性が向上するでしょうか？
- 適宜コメントを追加して処理を説明すると、コードがわかりやすくなります。
- 余裕があれば、正常系だけでなく、異常系も考えてみましょう。
  - 付属のテストケースが動作し、リングバッファとして正しく動作する範囲であれば、
    新たに関数を追加したり定義を変更してもかまいません。


みなさんの応募をお待ちしています！楽しんで！！

*/

#include <stdio.h>
#include <stdlib.h>

// リングバッファを表す構造体
struct RingBuffer {};

// リングバッファを生成して返す
struct RingBuffer *ring_new() {
  return NULL;
}

// リングバッファ ring の末尾に要素 value を追加
void ring_push(struct RingBuffer *ring, int value) {}

// リングバッファ ring の先頭要素を取り除いて返す
int ring_pop(struct RingBuffer *ring) { return 0; }

// テスト記述のための便利関数
int fail, success;
void assert_equal(int want, int got) {
  if (want == got) {
    printf("OK: want %d, got %d\n", want, got);
    success++;
  } else {
    printf("NG: want %d, got %d\n", want, got);
    fail++;
  }
}

// テストケース群
void test_push_pop() {
  struct RingBuffer *r = ring_new();
  ring_push(r, 2);
  ring_push(r, 3);

  assert_equal(2, ring_pop(r));
  assert_equal(3, ring_pop(r));
}

int main() {
  test_push_pop();
  // 他のテストケース呼び出しはここに追記

  if (fail != 0) {
    printf("FAILED: %d/%d tests failed\n", fail, fail + success);
    return EXIT_FAILURE;
  }
  printf("PASSED: All tests finished succesfully\n");
  return EXIT_SUCCESS;
}
